/*! **************************************************************************
 *	 Copyright (C) Siemens AG 2010 All Rights Reserved. Confidential
 *****************************************************************************
 *	 Written by SIEMENS - I MO	Confidential
 *****************************************************************************
 *
 *	 Projekt: SFC-Umrichter
 *
 *****************************************************************************
 *	\file
 *! \brief					 MPM-Anbindung
 *
 *! \author					H.Dethlefs
 * \ingroup pkgCCP_MES
 * Das \ref KMPM "Messsystem"  ist ...
 *
 ******************************************************************************/
#ifndef MPM_H
#define MPM_H
// Bitstrukturen
#include ".\..\..\..\..\XRG\inc\MPM20_Calc1.h"
#include ".\..\..\..\..\XRG\inc\MPM20_Calc2.h"
#include ".\..\..\..\..\XRG\inc\MPM21_Calc1.h"
#include ".\..\..\..\..\XRG\inc\MPM21_Calc2.h"


//! Enum der Zeitstufen KMPM
typedef enum E_MPM_Timer{
	kMpmTimer_none	= 0

	// Zeiten zu 3~ Netzseite
	, kMpmTimer_A_UNrequest		//!< maximale Zeitspanne zur Anforderung Netzspannung UAN
	, kMpmTimer_A_synchronize	//!< maximale Zeitspanne zur 3~Netz-Synchronisation
	, kMpmTimer_A_resynchronize //!< maximale Zeit fr eine Resynchronisierung nach Umschaltung auf PSM
	, kMpmTimer_A_UNFchange		//!< Wartezeit zum Einschwingen nach Wechsel zw. UAN und UAF

	, kMpmTimer_A_Umin1 //!< verzgerte 3~ Unterspannungsberwachung
	, kMpmTimer_A_Umin2 //!< verzgerte 3~ Unterspannungsberwachung
	, kMpmTimer_A_Umin3 //!< verzgerte 3~ Unterspannungsberwachung
	, kMpmTimer_A_Umin4 //!< verzgerte 3~ Unterspannungsberwachung
	, kMpmTimer_A_Umax1 //!< verzgerte 3~ berspannungsberwachung
	, kMpmTimer_A_Umax2 //!< verzgerte 3~ berspannungsberwachung
	, kMpmTimer_A_Umax3 //!< verzgerte 3~ berspannungsberwachung
	, kMpmTimer_A_Fuse	//!< UAF-Sicherungsberwachung

	, kMpmTimer_A_Fmax1 //!< verzgerte 3~ Maximalfrequenzberwachung
	, kMpmTimer_A_Fmax2 //!< verzgerte 3~ Maximalfrequenzberwachung
	, kMpmTimer_A_Fmin1 //!< verzgerte 3~ Minnimalfrequenzberwachung
	, kMpmTimer_A_Fmin2 //!< verzgerte 3~ Minnimalfrequenzberwachung

	/* Zeiten zu 1~ Bahnseite */
	, kMpmTimer_B_UNrequest	//!< maximale Zeitspanne zur Anforderung Netzspannung UBN
	, kMpmTimer_B_synchronize //!< maximale Zeitspanne zur 1~Netz-Synchronisation
	, kMpmTimer_B_UNFchange		//!< Wartezeit zum Einschwingen nach Wechsel zw. UBN und UBF

	, kMpmTimer_UB_releaseSupervision	//!< Freigabe B-Spannungsberwachung erfolgt, solange CCS keine Notfallbehandlungen durchfhrt
							
	, kMpmTimer_B_Umin1			//!< verzgerte 1~ Unterspannungsberwachung
	, kMpmTimer_B_Umin2			//!< verzgerte 1~ Unterspannungsberwachung
	, kMpmTimer_B_Umin3			//!< verzgerte 1~ Unterspannungsberwachung
	, kMpmTimer_B_Umax1			//!< verzgerte 1~ berspannungsberwachung
	, kMpmTimer_B_Umax2			//!< verzgerte 1~ berspannungsberwachung
	, kMpmTimer_B_Umax3			//!< verzgerte 1~ berspannungsberwachung
	, kMpmTimer_B_fault			//!< verzgerte 1~ Ungltigkeitsberwachung

	, kMpmTimer_B_Fmax1			//!< verzgerte 1~ Maximalfrequenzberwachung
	, kMpmTimer_B_Fmax2			//!< verzgerte 1~ Maximalfrequenzberwachung
	, kMpmTimer_B_Fmin1			//!< verzgerte 1~ Minnimalfrequenzberwachung
	, kMpmTimer_B_Fmin2			//!< verzgerte 1~ Minnimalfrequenzberwachung
	, kMpmTimer_B_F_releaseSupervision //!< Freigabe Frequenzberwachung erfolgt, solange CCS keine Notfallbehandlungen durchfhrt
	, kMpmTimer_B_THDmax		//!< Maximalzeit zur THD-berschreitung

	, kMpmTimer_A_Usync_VLT  //!< Umschaltung auf Synchronisierung mit Spannung vom Vorladetrafo
	, kMpmTimer_last
}E_MPM_Timer;

//! HW-Status der MPM-Schnittstellen
typedef struct validBits{
	uint16 uAE01		:1; //!< 01 Kanal defekt
	uint16 uAE02		:1; //!< 02 Kanal defekt
	uint16 uAE03		:1; //!< 03 Kanal defekt
	uint16 uAE04		:1; //!< 04 Kanal defekt
	uint16 uAE05		:1; //!< 05 Kanal defekt
	uint16 uAE06		:1; //!< 06 Kanal defekt
	uint16 uAE07		:1; //!< 07 Kanal defekt
	uint16 uAE08		:1; //!< 08 Kanal defekt
	uint16 uAE09		:1; //!< 09 Kanal defekt
	uint16 uSVbgt	:1; //!< 10 Kanal defekt
	uint16				:1; //	11
	uint16 mSynUN	:1; //!< 12 Synchronisiert auf UxN
	uint16 mOffset0	:1; //!< 13 Zustand Offsetabgleich mit 00 Null setzen	 , 01 Abgleich mit T1,
	uint16 mOffset1	:1; //!< 14									 10 Abgleich mit T2, 11 Stoppen / halten
	uint16 mPLL0		:1; //!< 15 Zustand Synchronisierung mit 00: Sperren (Vorsteuerung mit Nennfrequenz)
	uint16 mPLL1		:1; //!< 16 01: Normalbetrieb, 10: Starre Frequenz,	11: Halten
}validBits;

//! MSW_A Meldewort
typedef union UMPM_A_Messages_t {
	uint32 dw;
	struct {
		Bits_MPM20_MSW1 w1;
		Bits_MPM20_MSW2 w2;
	}msw;
}UMPM_A_Messages;

//! MSW_B Meldewort
typedef union UMPM_B_Messages_t {
	uint32 dw;
	struct {
	Bits_MPM21_MSW1 w1;
	Bits_MPM21_MSW2 w2;
	}msw;
}UMPM_B_Messages;


typedef enum eMPM_fault{
	kMPM_flt01			= 0x00000001,
	kMPM_flt02			= 0x00000002,
	kMPM_flt03			= 0x00000004,
	kMPM_flt04			= 0x00000008,
	kMPM_flt05			= 0x00000010,
	kMPM_flt06			= 0x00000020,
	kMPM_flt07			= 0x00000040,
	kMPM_flt08			= 0x00000080,
	kMPM_noNetMode	 = 0x00000100,
	kMPM_flt10			= 0x00000200,
	kMPM_flt11			= 0x00000400,
	kMPM_flt12			= 0x00000800,
	kMPM_flt13			= 0x00001000,
	kMPM_flt14			= 0x00002000,
	kMPM_flt15			= 0x00008000,
	kMPM_flt16			= 0x00010000,
	kMPM_flt17			= 0x00020000,
	kMPM_flt18			= 0x00040000,

	kMPM_noValidPPM	= 0x40000000, // keine Info aus Partnerprozessor
}eMPM_fault;
//! globale Fehler
typedef struct YMPM_gblFaults{
	uint32 uMessbaugruppe		:1; //!<  1 Messbaugruppe gestrt
	uint32 PI_E_A_MCB			:1; //!<  2 DC 110 V MCB TRIP
	uint16 uUsync_VLT			:1; //!<  3 Syn.-Spannung von Vorladetrafo nicht bereit
	uint32						:1; //    4
	uint32						:1; //    5
	uint32						:1; //    6
	uint32						:1; //    7
	uint32						:1; //    8
	uint32						:1; //    9
	uint32						:1; //   10
	uint32						:1; //   11
	uint32						:1; //   12
	uint32						:1; //   13
	uint32						:1; //   14
	uint32						:1; //   15
	uint32						:1; //   16
									     
	uint32 UBN_outOfRange		:1; //!< 17 UBN out of operational range
	uint32 Schwarzstart_N		:1; //!< 18 Schwarzstart anwhlen
	uint32 Schwarzstart_F		:1; //!< 19 Schwarzstart bestdigt nach CB closed
	uint32 uSchwarzstart_F		:1; //!< 20 Fehler im Schwarzstart (UBF ungleich UBN)
	uint32						:1; //   21
	uint32						:1; //   22
	uint32						:1; //   23
	uint32						:1; //   24
	uint32 UAN_outOfRange		:1; //!< 25 UAN out of operational range
	uint32						:1; //   26
	uint32 ePhaseShift_B		:1; //   27
	uint32						:1; //   28
	uint32 eShutOFFmax_UB		:1; //!< 29
	uint32 eShutOFFmin_UB		:1; //!< 30
	uint32 eShutOFFmax_UA		:1; //!< 31
	uint32 eShutOFFmin_UA		:1; //!< 32
}YMPM_gblFaults;
typedef union U_MPM_gblFaults{
	YMPM_gblFaults ib;
	int32		   iw;
}U_MPM_gblFaults;

//! CFC-Kommandowort am MPM-Eingangskonnektor
typedef struct YMPM_cfcCommand{
	uint16 cOFFSET			:2; //!<  1+2 Offsetabgleich 
	uint16 cUsync_VLT		:1; //!<  3 Syn.-Spannung von Vorladetrafo
	uint16 cUsync_NRM		:1; //    4
	uint16 cPLL_Hold		:1; //!<  5 PLL-Ansteuerung Halten zum CB-Schliessen
	uint16					:1; //    6 
	uint16					:1; //    7 
	uint16 cSuperviseFreq	:1; //!<  8 Frequenzberwachung freigeben
	uint16 cRequestUxN		:1; //!<  9 UxN anfordern
	uint16 cRequestUxF		:1; //!< 10 UxF anfordern
	uint16 pRequestUN		:1; //!< 11 UxN muss angefordert werden (Parameter)
	uint16 eUNallocated		:1; //!< 12 UxN auf Anforderung zugeteilt
	uint16 cSelectUN		:1; //!< 13 Befehl wechseln auf UxN
	uint16 cSelectUF		:1; //!< 14 Befehl wechseln auf UxF
	uint16 cSynchronize		:1; //!< 15 Befehl Synchronisierung starten
	uint16 eCBon			:1; //!< 16 Schalter ist geschlossen
}YMPM_cfcCommand;
typedef union U_MPM_CfcCommands{
	YMPM_cfcCommand ib;
	uint16		 iw;
}U_MPM_CfcCommands;

//! Auswahl Spannungsquelle
typedef enum eVoltageSelection{
	kUNselect = 1 //!< Auswahl UxN
,	kUFselect = 2 //!< Auswahl UxF
}eVoltageSelection;

//! Enum zur Anzeige der PLL-Stati
typedef enum EPLLstate {
	ePLL_blocked = 1 //!< PLL gesperrt
,	ePLL_async	 = 2 //!< PLL in asynchroner Betriebsart
,	ePLL_sync	 = 4 //!< PLL in synchroner Betriebsart
,	ePLL_hold	 = 8 //!< PLL halten
}EPLLstate;

//! MPM-Kommandowort
/*! Die ersten 16 Bits werden als Meldungen ausgegeben.
 *  Das Highwort beinhaltet interne Status.
 */
typedef struct YMPM_Command{
	uint16 cSelectOFFSET	:2; //!< 1-2 Offsetabgleich
	uint16					:1; //   3 
	uint16					:1; //   4
	uint16 cSelectPLL		:2; //!< 5-6 PLL-Synchronvorgaben
								/*!< 0: Sperren
									1: PLL Normalbetrieb
									2: PLL starre Frequenz
									3: Halten
								*/
	uint16 cSuperviseUBmin	:1; //!< 7 UB-Minimalberwachung freigeben
	uint16 cSuperviseFreq	:1; //!< 8 Frequenzberwachung aktivieren

	uint16 cRequestUmess	:2; //!< 9-10 Anforderung Synchronisierquelle	mit
								/*!< 01 Selektion 1 = UxN anfordern
								     10 Selektion 2 = UxF anfordern (UBN freigeben) 
								 */
	uint16 pRequestUN		:1; //!< 11 UxN muss angefordert werden	(Parameter)
	uint16 eUNallocated		:1; //!< 12 UxN auf Anforderung zugeteilt
	uint16 cSelectUmess		:2; //!< 13-14 Auswahl Synchronisierquelle	mit
								/*!< 01 Selektion 1 = wechseln auf UxN
									 10 Selektion 2 = wechseln auf UxF */

	uint16 cSynchronize		:1; //!< 15
	uint16 eCBon			:1; //!< 16 Schalterrckmeldung EIN

	uint16 cUsync_VLT		:1; //!< 17 Syn.-Spannung von Vorladetrafo
	uint16 eUsync_VLT		:1; //!< 18 Syn.-Spannung von Vorladetrafo ist gltig
	uint16					:1; //   19
	uint16					:1; //   20
	uint16					:1; //   21
	uint16					:1; //   22
	uint16					:1; //   23
	uint16					:1; //   24
	uint16					:1; //   25
	uint16					:1; //   26
	uint16					:1; //   27
	uint16					:1; //   28
	uint16					:1; //   29
	uint16					:1; //   30
	uint16					:1; //   31
	uint16					:1; //   32

}YMPM_Command;
typedef union U_MPM_Command{
	YMPM_Command ib;
	uint16		 iw[2];
	uint32		 dw;
}U_MPM_Command;

typedef enum eMPM_result{
	kMPM_eU_none			= 0x0 // Spannung unbestimmt
,	kMPM_eU_Valid			= 0x1 // Spannung gltig
,	kMPM_eU_Black			= 0x2 // Spannung schwarz
,	kMPM_eU_Fault			= 0x4 // Spannung nicht im erlaubten Bereich
}eMPM_result;
//! allg. Statuswort zu Messergebnissen
typedef struct YMPM_measurement_t{
	uint16 eUN				:4; //!< 01-04 Detailstatus UxN

	uint16 eUF				:4; //!< 05-08 Detailstatus UxF

	uint16 eUNallocated		:1; //!< 09 UxN zugeteilt (u.U.auf Anforderung )
	uint16					:1; //	10
	uint16 eUN_MCB_VT		:1; //!< 11 MCB Spannungserfassung UxN
	uint16 eUF_MCB_VT		:1; //!< 12 MCB Spannungserfassung UxF
	uint16 wf_min			:1; //!< 13 Warnung Frequenz unter Minimum
	uint16 wf_max			:1; //!< 14 Warnung Frequenz ber Maximum
	uint16 uf_min			:1; //!< 15 Strung Frequenz unter Minimum
	uint16 uf_max			:1; //!< 16 Strung Frequenz ber Maximum
}YMPM_measurement;

//! MPM Zustandswort Spannung
typedef struct YMPM_voltage_t{
	uint16 eUstate			:3; //!< 01-03 Detailstatus lt. eMPM_result
	uint16 uESpsm			:1; //!< Erdschluss in PSM der anderen Seite
	uint16 eU_1PhaseEarth	:1; //!< 05 1~ Erdschluss
	uint16 eU_fuse			:1; //!< 06 Sicherung Spannungserfassung
	uint16 eU_MCB_VT		:1; //!< 07 MCB Spannungserfassung (aus HW)
	uint16 uStrangPSM		:1; //!< 08 
	uint16 wShutOFFmin		:1; //!< 09 Unterspanunngswarnung
	uint16 uShutOFFmin		:1; //!< 10 Unterspanunngsabschaltung
	uint16 wShutOFFmax		:1; //!< 11 berspanunngswarnng
	uint16 uShutOFFmax		:1; //!< 12 berspanunngsabschaltung
	uint16 eSynchron		:1; //!< 13
	uint16 uNotSynchron		:1; //!< 14
	uint16 eUNtested		:1; //!< 15 UxN geprft
	uint16 eUNallocated		:1; //!< 16 UxN zugeteilt (u.U.auf Anforderung )
}YMPM_voltage;
typedef union UMPM_voltage_t{
	YMPM_voltage ib;
	uint16		 iw;
}UMPM_voltage;
//! MPM Zustandswort Strom
typedef struct YMPM_current_t{
	uint16					:1; //01
	uint16					:1; //02
	uint16					:1; //03
	uint16					:1; //04
	uint16					:1; //05
	uint16					:1; //06
	uint16					:1; //07
	uint16					:1; //08
	uint16 wTHDImax		:1; //!< 09 THD hat Maximum (mit Strombegrenzung) berschritten
	uint16					:1; //10
	uint16					:1; //11
	uint16					:1; //12
	uint16					:1; //13
	uint16					:1; //14
	uint16					:1; //15
	uint16					:1; //16
}YMPM_current;
typedef union UMPM_current{
	YMPM_current ib;
	uint16		 iw;
}UMPM_current;

//! Enum zu Frequenzstatus
typedef enum EMPM_Frequence{
	kMPM_f_none			 = 0x00 //!< Frequenz unbestimmt
,	kMPM_f_valid		 = 0x01 //!< Frequenz gltig
,	kMPM_wf_min			 = 0x10 //!< Warnung Frequenz unter Minimum
,	kMPM_wf_max			 = 0x20 //!< Warnung Frequenz ber Maximum
,	kMPM_uf_min			 = 0x40 //!< Strung Frequenz unter Minimum
,	kMPM_uf_max			 = 0x80 //!< Strung Frequenz ber Maximum
}EMPM_Frequence;
//! MPM Zustandswort Frequenz
typedef struct YMPM_Frequence_t{
	uint16 opcState		:4; //!< 01-04 Detailstatus zur Anzeige lt. eValueColor aus iOPC_Colors.h
	uint16 releaseUeb	:1; //!< 05 Frequenzberwachung mit Abschaltung freigegeben
	uint16					:1; //06
	uint16					:1; //07
	uint16					:1; //08
	uint16 eState			:8; //!< 09-16 berwachter Frequenzstatus lt EMPM_Frequence
}YMPM_Frequence;
typedef union UMPM_Frequence_t{
	YMPM_Frequence ib;
	uint16		 iw;
}UMPM_Frequence;

//! Verwaltungscontainer zu analogen Istwerte
typedef struct YSTATE_MEASUREMENT_t{
	uint16 values_selected	:1; //01 Container aktiv
	uint16 values_valid		:1; //02 Werte sind gltig
	uint16							:1; //03
	uint16							:1; //04
	uint16 commFault			:1; //05 Kommunikationsstrung zur Messstelle
	uint16							:1; //06
	uint16							:1; //07
	uint16							:1; //08
	uint16							:1; //09
	uint16							:1; //10
	uint16							:1; //11
	uint16							:1; //12
	uint16							:1; //13
	uint16							:1; //14
	uint16							:1; //15
	uint16							:1; //16
}YSTATE_MEASUREMENT;

typedef struct YRA{
	YSTATE_MEASUREMENT measState;
	int16 eVB1	; // Gltigkeitsbits Datenwort-1 Valid Bit Data Word-1
	int16 eVB2	; // Gltigkeitsbits Datenwort-2 (System-Code) Valid Bit Data Word-2 (System-Code)
	int16 rNAM	;
	int16 eMSW	; // berwachungs-Zustandswort Monitoring Status Word
	float xUAF_mp; // Spannung 3P-Einspeisung Betrag xUAF / Mitsystem
	float xUAF_mn; // Spannung 3P-Einspeisung Betrag xUAF / Gegensystem
	float xUAF1	; // Spannung 3P-Einspeisung Phase 1 / Gesamt-Effektivwert
	float xUAF2	; // Spannung 3P-Einspeisung Phase 2 / Gesamt-Effektivwert
	float xUAF3	; // Spannung 3P-Einspeisung Phase 3 / Gesamt-Effektivwert
	float xIAF_mp; // Strom 3P-Einspeisung Betrag xIAF / Mitsystem
	float xIAF_mn; // Strom 3P-Einspeisung Betrag xIAF / Gegensystem
	float xIAF1	; // Strom 3P-Einspeisung Phase 1/ Gesamt-Effektivwert
	float xIAF2	; // Strom 3P-Einspeisung Phase 2/ Gesamt-Effektivwert
	float xIAF3	; // Strom 3P-Einspeisung Phase 3/ Gesamt-Effektivwert
	float xPA	 ; // Wirkleistung 3P-Einspeisung	/ Mitsystem
	float xQA	 ; // Blindleistung 3P-Einspeisung / Mitsystem
}YRA;

//! gelesene Binreingnge an MPM20 (A-seitiges MessSystem)
typedef union UMPM20_VB1 {
	uint16 iw;
	struct YMPM20_VB1 {
		uint16 eUAN_12_RDY		:1; //!<  1 Messumformer 1 bereit	
		uint16 eUAN_23_RDY		:1; //!<  2 Messumformer 2 bereit	
		uint16 eUAN_31_RDY		:1; //!<  3 Messumformer 3 bereit	
		uint16 eUAF_1_RDY		:1; //!<  4 Messumformer 4 bereit	
		uint16 eUAF_2_RDY		:1; //!<  5 Messumformer 5 bereit	
		uint16 eUAF_3_RDY		:1; //!<  6 Messumformer 6 bereit	
		uint16 eIAF_1_RDY		:1; //!<  7 Messumformer 7 bereit	
		uint16 eIAF_2_RDY		:1; //!<  8 Messumformer 8 bereit	
		uint16 eIAF_3_RDY		:1; //!<  9 Messumformer 9 bereit	
		uint16 ePWR_SUPPLY_OK	:1; //!< 10 MPM Stromversorgung OK
		uint16 eDCF77			:1; //!< 11 MPM Zeitsynchronisierung
		uint16 cVLT_SEL			:1; //!< 12 Auswahl SYNC_Spannung (A_C_SYN_SEL)
		uint16 cOFS_SEL_0		:1; //!< 13 Offsetabgleich Bit0 (J_C_OFS_0)
		uint16 cOFS_SEL_1		:1; //!< 14 Offsetabgleich Bit1 (J_C_OFS_1)
		uint16 cSYN_SEL_0		:1; //!< 15 Auswahl PLL Bit0 (A_C_PLL_0)
		uint16 cSYN_SEL_1		:1; //!< 16 Auswahl PLL Bit1 (A_C_PLL_1) 
	}ib;
}UMPM20_VB1;

//! gelesene Binreingnge an MPM21 (B-seitiges MessSystem)
typedef union UMPM21_VB1 {
	uint16 iw;
	struct YMPM21_VB1 {
		uint16 eUBN_RDY			:1; //!<  1 Messumformer 1 bereit	
		uint16 eUBF_RDY			:1; //!<  2 Messumformer 2 bereit	
		uint16 eIBF_RDY			:1; //!<  3 Messumformer 3 bereit	
		uint16 eICP_1_RDY		:1; //!<  4 Messumformer 4 bereit	
		uint16 eICP_2_RDY		:1; //!<  5 Messumformer 5 bereit	
		uint16 eICP_3_RDY		:1; //!<  6 Messumformer 6 bereit	
		uint16 eICN_1_RDY		:1; //!<  7 Messumformer 7 bereit	
		uint16 eICN_2_RDY		:1; //!<  8 Messumformer 8 bereit	
		uint16 eICN_3_RDY		:1; //!<  9 Messumformer 9 bereit	
		uint16 ePWR_SUPPLY_OK	:1; //!< 10 MPM Stromversorgung OK
		uint16 eDCF11			:1; //!< 11 MPM Zeitsynchronisierung
		uint16 cVLT_SEL			:1; //!< 12 Auswahl SYNC_Spannung (B_C_SYN_SEL)
		uint16 cOFS_SEL_0		:1; //!< 13 Offsetabgleich Bit0 (J_C_OFS_0)
		uint16 cOFS_SEL_1		:1; //!< 14 Offsetabgleich Bit1 (J_C_OFS_1)
		uint16 cSYN_SEL_0		:1; //!< 15 Auswahl PLL Bit0 (B_C_PLL_0)
		uint16 cSYN_SEL_1		:1; //!< 16 Auswahl PLL Bit1 (B_C_PLL_1) 
	}ib;
}UMPM21_VB1;
//! 3~Istwerte
typedef struct YMPM_XA{
	YSTATE_MEASUREMENT measState;
	UMPM20_VB1 eVB; //!< Gltigkeitsbits Datenwort-1 Valid Bit Data Word-1
//	int16 eVB1; //!< Gltigkeitsbits Datenwort-1 Valid Bit Data Word-1
//	int16 eVB2; //!< Gltigkeitsbits Datenwort-2 (System-Code) Valid Bit Data Word-2 (System-Code)
	int16 rNAM;
//	int16 eMSW;	 //!< berwachungs-Zustandswort Monitoring Status Word
	UMPM_A_Messages eMSW;	 //!< berwachungs-Zustandswort Monitoring Status Word
	float xUAF_mp; //!< Spannung 3P-Einspeisung Betrag xUAF / Mitsystem
	float xUAF_mn; //!< Spannung 3P-Einspeisung Betrag xUAF / Gegensystem
	float xUAF1	; //!< Spannung 3P-Einspeisung Phase 1 / Gesamt-Effektivwert
	float xUAF2	; //!< Spannung 3P-Einspeisung Phase 2 / Gesamt-Effektivwert
	float xUAF3	; //!< Spannung 3P-Einspeisung Phase 3 / Gesamt-Effektivwert
	float xIAF_mp; //!< Strom 3P-Einspeisung Betrag xIAF / Mitsystem
	float xIAF_mn; //!< Strom 3P-Einspeisung Betrag xIAF / Gegensystem
	float xIAF1	; //!< Strom 3P-Einspeisung Phase 1/ Gesamt-Effektivwert
	float xIAF2	; //!< Strom 3P-Einspeisung Phase 2/ Gesamt-Effektivwert
	float xIAF3	; //!< Strom 3P-Einspeisung Phase 3/ Gesamt-Effektivwert
	float xPA	 ; //!< Wirkleistung 3P-Einspeisung	/ Mitsystem
	float xQA	 ; //!< Blindleistung 3P-Einspeisung / Mitsystem

	float xUAF;		//!< Effektivwert
	float xIAF;		//!< Effektivwert
	float xUAN1;	 //!< 110kV / 50 Hz busbar voltage L1
	float xUAN2;	 //!< 110kV / 50 Hz busbar voltage L2
	float xUAN3;	 //!< 110kV / 50 Hz busbar voltage L3

	float xUAN;		//!< Istspannung UAN
	float xfA;		//!< Istfrequenz
	float xSA;		//!< berechnete Scheinleistung
	float xCos_phi; //!< Wirkungsgrad
} YMPM_XA;

typedef struct YRB{
	int16 eVB1; // Gltigkeitsbits Datenwort-1 Valid Bit Data Word-1
	int16 eVB2; // Gltigkeitsbits Datenwort-2 (System-Code) Valid Bit Data Word-2 (System-Code)
	int16 xIB_FILTER; // B-Filter Effektivstrom
	int16 eMSW; // berwachungs-Zustandswort Monitoring Status Word
	float xUBF_m	; // Spannung 1P-Einspeisung Betrag xUBF / Grundschwingung Voltage 1P-Feeder Magnitude xUBF / Fundamental oscillation
	float xUBF__r	; // Spannung 1P-Einspeisung/ Gesamt-Effektivwert Voltage 1P-Feeder/ Total rms-value
	float xIBF_m	; // Strom 1P-Einspeisung Betrag xIBF/ Grundschwingung Current 1P-Feeder Magnitude xIBF/ Fundamental oscillation
	float xIBF__r	; // Strom 1P-Einspeisung/ Gesamt-Effektivwert Current 1P-Feeder/ Total rms-value
	float xPBF		; // Wirkleistung 1P-Einspeisung/ Grundschwingung Active Power 1P-Feeder/ Fundamental oscillation
	float xQBF		; // Blindleistung 1P-Einspeisung/ Grundschwingung Reactive Power 1P-Feeder/ Fundamental oscillation
	float xUBF_1r	; // Spannung 1P-Einspeisung:/ Verhltnis: Grundschwingungs- / Gesamt-Effektivwert Voltage 1P-Feeder:/ Ratio: Fundamental oscillation / Total rms-value
	float xUBF_2r	; // Spannung 1P-Einspeisung:/ Verhltnis: Grundschwingungs- / Gesamt-Effektivwert Voltage 1P-Feeder:/ Ratio: Fundamental oscillation / Total rms-value
	float xICsum_r	; // Strom 1P-Einspeisung:/ Verhltnis: Grundschwingungs- / Gesamt-Effektivwert Current 1P-Feeder:/ Ratio: Fundamental oscillation / Total rms-value
	float rNBF_THDI; // Strom 1P-Einspeisung/ Gesamte harmonische Verzerrung Current 1P-Feeder/ Total Harmonic Distortion (THD)
}YRB;

//! 1~Istwerte
typedef struct YMPM_XB{ // Istwerte 1~ Bahnseite
	YSTATE_MEASUREMENT measState;
	UMPM21_VB1 eVB  ; //!< Gltigkeitsbits Datenwort-2 Valid Bit Data Word-2
//	int16 eVB1;		//!< Gltigkeitsbits Datenwort-1 Valid Bit Data Word-1
//	int16 eVB2;		//!< Gltigkeitsbits Datenwort-2 (System-Code) Valid Bit Data Word-2 (System-Code)
	int16 xIB_FILTER; //!< B-Filter Effektivstrom
//	int16 eMSW		; //!< berwachungs-Zustandswort Monitoring Status Word
	UMPM_B_Messages eMSW		; //!< berwachungs-Zustandswort Monitoring Status Word
	float xUBF_m	; //!< Spannung 1P-Einspeisung Betrag xUBF / Grundschwingung 
	float xUBF_1r	; //!< Spannung 1P-Einspeisung/ Gesamt-Effektivwert 1
	float xUBF_2r	; //!< Spannung 1P-Einspeisung: Gesamt-Effektivwert 2
	float xUBF_r	; //!< Spannung 1P-Einspeisung/ Gesamt-Effektivwert
	float xIBF_m	; //!< Strom 1P-Einspeisung Betrag xIBF/ Grundschwingung
	float xIBF_r	; //!< Strom 1P-Einspeisung/ Gesamt-Effektivwert
	float xPBF		; //!< Wirkleistung 1P-Einspeisung/ Grundschwingung Active
	float xQBF		; //!< Blindleistung 1P-Einspeisung/ Grundschwingung
	float xICsum_r	; //!< Strom 1P-Einspeisung:/ Verhltnis: Grundschwingungs- / Gesamt-Effektivwert
	float rNBF_THDI	; //!< Strom 1P-Einspeisung/ Gesamte harmonische Verzerrung

//! bahnseitige Istwerte
	float xUBN	 ; //!< Istwert UBN
	float xUBF	 ; //!< Istwert UBF
	float xIBF	 ; //!< Istwert IBF
	float xUBFF	 ; //!< Istwert
	float xIBFF	 ; //!< Istwert
	float xTHDibf; //!< Istwert
//! Umrichterausgang
	float xUB; //!< Istspannung UBF
	float xfB; //!< Istfrequenz
	float xPB; //!< Istwirkleistung
	float xQB; //!< Istblindleistung
	float xSB; //!< Istscheinleistung

	float xCos_phi;
} YMPM_XB;

typedef struct YMPM_EIN{
	U_MPM_Command	 cA_yard;
	U_MPM_Command	 cB_yard;
}YMPM_EIN;

//! InterprozessKommunikation
typedef struct YMPM_IPC1{
	U_MPM_Command		cA_yard;
	U_MPM_Command		cB_yard;
	YMPM_measurement	statusA;
	YMPM_measurement	statusB;
}YMPM_IPC1;

typedef struct YMPM_IPC2{
	// Istwerte 3~ Netzseite
	YMPM_XA			XA;
	uint16			statusUAN;
	UMPM_voltage	xFaultsUAN;	 //!< aktives Fehlerwort
	uint16			statusUAF;
	UMPM_voltage	xFaultsUAF;	 //!< aktives Fehlerwort
	uint16			statusIA;
	UMPM_current	xFaultsIA;	//!< aktives Fehlerwort xIA
	uint16			statusFA;
	UMPM_Frequence	xFaultsFA;	//!< aktives Fehlerwort xFA
	// Istwerte 1~ Bahnseite
	YMPM_XB			XB;
	uint16			statusUBN;
	UMPM_voltage	xFaultsUBN;	 //!< aktives Fehlerwort
	uint16			statusUBF;
	UMPM_voltage	xFaultsUBF;	 //!< aktives Fehlerwort
	uint16			statusIB ;
	UMPM_current	xFaultsIB;	//!< aktives Fehlerwort xIB
	uint16			statusFB ;
	UMPM_Frequence	xFaultsFB;	//!< aktives Fehlerwort xFB

		UMPM_A_Messages	mMPMA; //!< MPM-Meldungen (aus Controlbus)
	UMPM_B_Messages	mMPMB; //!< MPM-Meldungen (aus Controlbus)
	u_int32			eVB0;
	u_int32			eVB1;
}YMPM_IPC2;

typedef enum EMPM_test{
	  kUA_K_LE_3AC = 1
	, kUB_K_LE_1AC

	, kMPM_lastTestValue
}EMPM_test;

// Testausgaben
typedef struct YMPM_test{
	float value[kMPM_lastTestValue];
	uint32 status[kMPM_lastTestValue];
}YMPM_test;
//! Gesamtstruktur KMPM
typedef struct ZMPM_IO {
//! SFC-Modul zur Anmeldung am berlagerten Board
	YSfcModule cfgBoard;
// !!!NICHT VERSCHIEBEN ODER NDERN!!!

	YAK_MPM				iAK;						//!< Anlagenparameter
	YSFCTimer			timer[kMpmTimer_last];//!< Zeitstufen

	YMPM_EIN			Ein;		//!< Eingangsschnittstelle Peripherie
	UMPM_A_Messages		mMPMA; //!< MPM-Meldungen (aus Controlbus)
	UMPM_B_Messages		mMPMB; //!< MPM-Meldungen (aus Controlbus)
	u_int32				eVB0;		//!< MPM-Meldewort zur Messwerterfassung
	u_int32				eVB1;		//!< MPM-Meldewort zur Messwerterfassung

	U_MPM_gblFaults		faults;	   //!< allgemeiner Fehlerstatus
	cFault32			cFaults; //!< eigenes Fehlermodul zur Fehlerpriorisierung und Meldeausgabe
// 3~ Netzseite
	U_MPM_Command		cA_yard;	//!< A-Kommandowort
	YMPM_XA				XA;			//!< A-Istwerte aus MPM
	YMPM_measurement	statusA;	//!< allg.A-Status

	cFault16			faultUAN;
	UMPM_voltage		statusUAN;	//!< Statuswort xUAN
	UMPM_voltage		xFaultsUAN;	 //!< aktives Fehlerwort xUAN
	cFault16			faultUAF;
	UMPM_voltage		statusUAF;	//!< Statuswort xUAF
	UMPM_voltage		xFaultsUAF;	 //!< aktives Fehlerwort xUAF
	cFault16			faultIA;
	UMPM_current		statusIA;	//!< Statuswort xIA
	UMPM_current		xFaultsIA;	//!< aktives Fehlerwort xIA
	cFault16			faultFA;
	UMPM_Frequence		statusFA;	//!< Statuswort xFA
	UMPM_Frequence		xFaultsFA;	//!< aktives Fehlerwort xFA
	u_int16				statusPLL_A;//!< Statuswort PLL A
	cFault16			msgPLL_A;
// 1~ Bahnseite
	U_MPM_Command		cB_yard;	//!< B-Kommandowort
	YMPM_XB				XB;			//!< B-Istwerte aus MPM
	YMPM_measurement	statusB;	//!< allg.B-Status
	cFault16			faultUBN ;
	UMPM_voltage		statusUBN;	//!< Statuswort xUBN
	UMPM_voltage		xFaultsUBN;	 //!< aktives Fehlerwort xUBN
	cFault16			faultUBF ;
	UMPM_voltage		statusUBF;	//!< Statuswort xUBF
	UMPM_voltage		xFaultsUBF;	 //!< aktives Fehlerwort xUBF
	cFault16			faultIB  ;
	UMPM_current		statusIB;	//!< Statuswort xIB
	UMPM_current		xFaultsIB;	//!< aktives Fehlerwort xIB
	cFault16			faultFB  ;
	UMPM_Frequence		statusFB;	//!< Statuswort xFB
	UMPM_Frequence		xFaultsFB;	//!< aktives Fehlerwort xFB
	u_int16				statusPLL_B;//!< Statuswort PLL B
	cFault16			msgPLL_B;

	float xLoadFactor; //!< Lastfaktor
	float xEfficiency; //!< Effizienz

	float xI_B9		; // gegltteter Istwert

// Glttungszeitkonstante
	float Tsmooth_B9 ;
	float Tsmooth_LF ; // Loadfactor
	float Tsmooth_EF ; // Efficiency

	int wVarNet;
	int	prozNr;
	float Tabtast;
	YMPM_IPC1 ipc1;
	YMPM_IPC2 ipc2;

	uint16 test;
	YMPM_test ytest;
}ZMPM_IO;

#endif
